==> lion-pytorch-0.0.1.tar.gz: lion-pytorch-0.0.1/lion_pytorch/lion_pytorch.py <==
from typing import Tuple, Optional, Callable

import torch
from torch.optim.optimizer import Optimizer

# functions

def exists(val):
    return val is not None

def inplace_ema(old, val, beta):
    old.mul_(beta).add_(val, alpha = 1 - beta)

# class

class Lion(Optimizer):
    def __init__(
        self,
        params,
        lr: float = 1e-4,
        betas: Tuple[float, float] = (0.9, 0.99),
        weight_decay: float = 0.0
    ):
        assert lr > 0.
        assert all([0. <= beta <= 1. for beta in betas])

        defaults = dict(
            lr = lr,
            betas = betas,
            weight_decay = weight_decay
        )

        super().__init__(params, defaults)

    @torch.no_grad()
    def step(
        self,
        closure: Optional[Callable] = None
    ):

        loss = None
        if exists(closure):
            with torch.enable_grad():
                loss = closure()

        for group in self.param_groups:
            for p in group['params']:

                grad = p.grad

                if not exists(grad):
                  continue

                state = self.state[p]
                lr, wd, beta1, beta2 = group['lr'], group['weight_decay'], *group['betas']

                # stepweight decay

                p.data.mul_(1 - lr * wd)

                # init state - exponential moving average of gradient values

                if len(state) == 0:
                    state['exp_avg'] = torch.zeros_like(p)

                exp_avg = state['exp_avg']

                # weight update

                update = exp_avg.clone()
                inplace_ema(update, grad, beta1)
                p.add_(torch.sign(update), alpha = -lr)

                # decay the momentum running average coefficient

                inplace_ema(exp_avg, grad, beta2)

        return loss

==> lion-pytorch-0.0.2.tar.gz: lion-pytorch-0.0.2/lion_pytorch/lion_pytorch.py <==
from typing import Tuple, Optional, Callable

import torch
from torch.optim.optimizer import Optimizer

# functions

def exists(val):
    return val is not None

# class

class Lion(Optimizer):
    def __init__(
        self,
        params,
        lr: float = 1e-4,
        betas: Tuple[float, float] = (0.9, 0.99),
        weight_decay: float = 0.0
    ):
        assert lr > 0.
        assert all([0. <= beta <= 1. for beta in betas])

        defaults = dict(
            lr = lr,
            betas = betas,
            weight_decay = weight_decay
        )

        super().__init__(params, defaults)

    @torch.no_grad()
    def step(
        self,
        closure: Optional[Callable] = None
    ):

        loss = None
        if exists(closure):
            with torch.enable_grad():
                loss = closure()

        for group in self.param_groups:
            for p in group['params']:

                grad = p.grad

                if not exists(grad):
                  continue

                state = self.state[p]
                lr, wd, beta1, beta2 = group['lr'], group['weight_decay'], *group['betas']

                # stepweight decay

                p.data.mul_(1 - lr * wd)

                # init state - exponential moving average of gradient values

                if len(state) == 0:
                    state['exp_avg'] = torch.zeros_like(p)

                exp_avg = state['exp_avg']

                # weight update

                update = exp_avg.clone().lerp_(grad, 1 - beta1)
                p.add_(torch.sign(update), alpha = -lr)

                # decay the momentum running average coefficient

                exp_avg.lerp_(grad, 1 - beta2)

        return loss
